/**
 * Created by janis dähne.
 */
import {
  RESET_AfterTestResultsAction,
  RESET_TestResultsAction,
  ResetAction, SET_addCustomTestAssetFilesModalFilterTextAction, SET_addCustomTestAssetFilesModalUploadPercentageAction,
  SET_botLeftPanelViewSizeAction,
  SET_botRightPanelViewSizeAction,
  SET_bottomLeftSelectedTabIndexAction,
  SET_bottomRightSelectedTabIndexAction,
  SET_dummyAction,
  SET_exercisePreviewFrontendAction,
  SET_hDividerPosOffsetAction, SET_isAddCustomTestAssetFilesModalDisplayedAction,
  SET_isCodeEditorSettingsDialogDisplayedAction,
  SET_isCustomTestDialogDisplayedAction,
  SET_isJustRunProgramArgsInputDisplayedAction,
  SET_isLeftTimeStringDisplayedAction,
  SET_isLoadingAction,
  SET_isOnGotFocusInterceptionOverlayDisplayedAction,
  SET_isViewTestDialogDisplayedAction,
  SET_justRunProgramArgsAction, SET_lastKnownEditorContentAction,
  SET_selectedConsoleViewTestAction,
  SET_selectedPLangIdAction,
  SET_solutionAction,
  SET_topLeftPanelViewSizeAction,
  SET_topLeftSelectedTabIndexAction,
  SET_topRightPanelViewSizeAction,
  SET_topRightSelectedTabIndexAction,
  SET_vDividerPosOffsetAction
} from "../../reducers/doExerciseSite/doExerciseSiteReducer";
import {ActionType} from "../../reducers/doExerciseSite/doExerciseSiteActionTypes";
import {AwaitActions, MultiActions} from "../types";
import {
  getAllDoExerciseAfterTestResults,
  getDoExerciseAfterSubmitTestResults,
  getDoExerciseAllTestResults,
  getDoExerciseAsync,
  getDoExerciseSolutionAsync,
  getDoExerciseSubmitTestResults,
  resetUserSolutionDoExerciseSiteAsync
} from "./doExerciseCrudActions";
import {getTestTypesAsync} from "../testTypesActions";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {
  SET_mainFileIdAction,
  SET_solutionFilesAction
} from "../../reducers/doExerciseSite/subSets/setDoExerciseSolutionReducer";
import {SolutionDoExerciseFullBase, SolutionFileDoExerciseFullBase} from "../../../types/exerciseSolution";
import {setSelectedTabIdDoEditor} from "./editorActions/doEditorActions";
import {ExercisePreviewsFrontendExclusive} from "../../../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {createAndRefreshAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {
  SET_afterMainFileIdAction,
  SET_afterSolutionFilesAction,
  SET_afterSolutionFirstDummyAction
} from "../../reducers/doExerciseSite/subSets/setAfterSolutionReducer";
import {setAfterEditor_selectedTabIdDoEditor} from "./editorActions/afterEditorActions";
import {
  mergeCustomTestResults,
  mergeNormalTestResults,
  mergeSubmitTestResults
} from "../../../helpers/convertersAndTransformers";
import {setTests} from "./sub/testActions";
import {setAfterNormalTests} from "./sub/afterNormalTestActions";
import {setAfterSubmitTests} from "./sub/afterSubmitTestActions";
import {setCustomTests} from "./sub/customTestActions";
import {
  AceEditorsStorageHelper,
  AceStoredTabLike
} from '../../../components/codeEditors/aceEditors/aceEditorsStorageHelper'
import {
  doEditorAfterEditorPanelViewEditorId,
  doEditorEditorPanelViewEditorId,
  tutorViewEditorPanelViewEditorId
} from '../../../constants'
import {PLangFullBase} from '../../../types/pLangs'
import Logger from '../../../helpers/logger'
import {_createFirstDummyAfterSolution, getDoExerciseAfterSolutionAsync} from './afterSolutionCrudActions'
import {AfterSolutionDoExerciseFullBase} from '../../../types/exerciseAfterSolution'
import {setAfterCustomTests} from './sub/afterCustomTestActions'
import {getSystemSettingsAsync} from '../systemSettingsSite/systemSettingsCrudActions'
import {
  getGlobalSystemSettingsAsync,
  getTestServerPublicSettingsAsync
} from '../globalSystemSettings/globalSystemSettingsCrudActions'
import {getPLangsAsync} from '../plangSettingsView/pLangsCrudActions'


export function setIsLeftTimeStringDisplayed(isLeftTimeStringDisplayed: boolean): SET_isLeftTimeStringDisplayedAction {
  return {
    type: ActionType.SET_isLeftTimeStringDisplayed,
    isLeftTimeStringDisplayed
  }
}

/**
 *
 * @param {ExercisePreviewsFrontendExclusive} exercisePreviewFrontendTemp
 * @returns {SET_exercisePreviewFrontendAction}
 */
export function setExercisePreviewFrontendDoExercise(exercisePreviewFrontendTemp: ExercisePreviewsFrontendExclusive | null): SET_exercisePreviewFrontendAction {
  return {
    type: ActionType.SET_exercisePreviewFrontend,
    exercisePreviewFrontendTemp: exercisePreviewFrontendTemp
  }
}


//-- normal solution
export function setSolutionFiles(solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>): SET_solutionFilesAction {
  return {
    type: ActionType.SET_solutionFiles,
    solutionFiles
  }
}

export function setMainFileIdDoExercise(mainFileId: number): SET_mainFileIdAction {
  return {
    type: ActionType.SET_mainFileId,
    mainFileId
  }
}

//--- after solution

export function setAfterSolutionFirstDummy(afterSolution: AfterSolutionDoExerciseFullBase): SET_afterSolutionFirstDummyAction {
  return {
    type: ActionType.SET_afterSolutionFirstDummy,
    afterSolution
  }
}

export function setAfterSolutionFiles(solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>): SET_afterSolutionFilesAction {
  return {
    type: ActionType.SET_afterSolutionFiles,
    solutionFiles
  }
}

export function setAfterMainFileId(mainFileId: number): SET_afterMainFileIdAction {
  return {
    type: ActionType.SET_afterMainFileId,
    mainFileId
  }
}

export function setSelectedPLangId(selectedPLangId: number): SET_selectedPLangIdAction {
  return {
    type: ActionType.SET_selectedPLangId,
    selectedPLangId
  }
}


export function resetDoExerciseSolutionToTemplate(releaseCode: string, currentPLangId: number,
                                                  allPlangs: ReadonlyArray<PLangFullBase>
): MultiActions {
  return async (dispatch, getState) => {


    await dispatch(resetUserSolutionDoExerciseSiteAsync(releaseCode, currentPLangId))


    const pLang = allPlangs.find(p => p.id === currentPLangId)

    if (!pLang) {
      const msg = `could not find plang with id ${currentPLangId}`
      Logger.error(msg)
      throw new Error(msg)
    }

    const solution = getState().doExerciseSiteState.solution

    //refresh ace storage because edit is already mounted we need to do this
    AceEditorsStorageHelper.removeEditor(doEditorEditorPanelViewEditorId)

    const storedTabs = solution.solutionFiles.map<AceStoredTabLike>(
      p => {
        return {
          id: p.id,
          content: p.content,
          language: pLang.internalName
        }
      })
    AceEditorsStorageHelper.addEditorWithTabs(doEditorEditorPanelViewEditorId, storedTabs)


    //try to select the first displayed tab (if any)
    if (solution.solutionFiles.length > 0
      && solution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
      dispatch(setSelectedTabIdDoEditor(
        solution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
    }

    //also reset the test results in frontend (backend is already set)
    dispatch(resetNormalTestResults())
    dispatch(resetCustomTestResults())

  }
}

//obsolete because we need to set the after solution back to the solution but we
//need the ids... so we can't forget these ... maybe there is an easy solution for this problem
//but for now just comment it out maybe do this on the backend ...??
// export function resetAfterEditor(allPLangs: ReadonlyArray<PLangFullBase>,
//                                  solution: SolutionDoExerciseFullBase
// ): MultiActions {
//   return dispatch => {
//
//
//     dispatch(setAfterSolutionFiles([...solution.solutionFiles]))
//
//     const pLang = allPLangs.find(p => p.id === solution.pLangId)
//
//     if (!pLang) {
//       Logger.error('could not find plang')
//     }
//
//     //set the editor storage
//     //add a dummy because we have no access to the real editor tabs... but we only need the ids
//     // noinspection TsLint
//     const storedTabs = solution.solutionFiles.map<AceStoredTabLike>(p => {
//       return {
//         id: p.id,
//         content: p.content,
//         language: pLang.internalName
//       }
//     })
//     AceEditorsStorageHelper.removeEditor(doEditorAfterEditorPanelViewEditorId)
//     AceEditorsStorageHelper.addEditorWithTabs(doEditorAfterEditorPanelViewEditorId, storedTabs)
//
//     //this will force the editor to rerender and update the session
//     dispatch(setAfterMainFileId(solution.mainFileId))
//
//     //also reset after test results
//     dispatch(resetAfterNormalTestResults())
//     dispatch(resetAfterSubmitTestResults())
//
//   }
// }

//resets normal, custom tests

/**
 * this resets the normal test results, also resets the custom test results
 * and clears the selected test
 */
export function resetDoExerciseTestResults(): MultiActions {
  return dispatch => {

    dispatch(_resetDoExerciseTestResults())
    dispatch(setSelectedConsoleViewTest(null))

  }
}

/**
 * this resets all after test results (normal/custom/submit)
 */
export function resetDoExerciseAfterTestResults(): MultiActions {
  return dispatch => {

    dispatch(_resetDoExerciseAfterTestResults())
    dispatch(setSelectedConsoleViewTest(null))

  }
}

/**
 * this resets the normal test results, also resets the custom test results
 */
export function _resetDoExerciseTestResults(): RESET_TestResultsAction {
  return {
    type: ActionType.RESET_TestResults,
  }
}

export function _resetDoExerciseAfterTestResults(): RESET_AfterTestResultsAction {
  return {
    type: ActionType.RESET_AfterTestResults,
  }
}


//-- reset test results

export function resetNormalTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setTests(mergeNormalTestResults(getState().doExerciseSiteState.exercise.tests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function resetCustomTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setCustomTests(mergeCustomTestResults(getState().doExerciseSiteState.exercise.customTests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function resetAfterNormalTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setAfterNormalTests(mergeNormalTestResults(getState().doExerciseSiteState.afterNormalTests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function resetAfterSubmitTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setAfterSubmitTests(mergeSubmitTestResults(getState().doExerciseSiteState.afterSubmitTests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function resetAfterCustomTestResults(): MultiActions {
  return (dispatch, getState) => {
    //getState().doExerciseSiteState.exercise.customTests should be the same as the after tests anyway
    dispatch(setAfterCustomTests(mergeCustomTestResults(getState().doExerciseSiteState.exercise.customTests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function setSelectedConsoleViewTest(selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive
  | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null): SET_selectedConsoleViewTestAction {
  return {
    type: ActionType.SET_selectedConsoleViewTest,
    selectedConsoleViewTest
  }
}

//--- grid system, tab indices
export function setTopRightSelectedTabIndex(topRightSelectedTabIndex: number): SET_topRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_topRightSelectedTabIndex,
    topRightSelectedTabIndex
  }
}

export function setTopLeftSelectedTabIndex(topLeftSelectedTabIndex: number): SET_topLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_topLeftSelectedTabIndex,
    topLeftSelectedTabIndex
  }
}

export function setBottomRightSelectedTabIndex(bottomRightSelectedTabIndex: number): SET_bottomRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomRightSelectedTabIndex,
    bottomRightSelectedTabIndex
  }
}

export function setBottomLeftSelectedTabIndex(bottomLeftSelectedTabIndex: number): SET_bottomLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomLeftSelectedTabIndex,
    bottomLeftSelectedTabIndex
  }
}


//--- gripper
export function setDoExerciseHDividerPosOffset(hDividerPosOffset: number): SET_hDividerPosOffsetAction {
  return {
    type: ActionType.SET_hDividerPosOffset,
    hDividerPosOffset
  }
}

export function setDoExerciseVDividerPosOffset(vDividerPosOffset: number): SET_vDividerPosOffsetAction {
  return {
    type: ActionType.SET_vDividerPosOffset,
    vDividerPosOffset
  }
}

//--- panel view sizes

export function setTopRightPanelViewSize(topRightPanelViewSize: PanelViewSize): SET_topRightPanelViewSizeAction {
  return {
    type: ActionType.SET_topRightPanelViewSize,
    topRightPanelViewSize
  }
}

export function setTopLeftPanelViewSize(topLeftPanelViewSize: PanelViewSize): SET_topLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_topLeftPanelViewSize,
    topLeftPanelViewSize
  }
}

export function setBotRightPanelViewSize(botRightPanelViewSize: PanelViewSize): SET_botRightPanelViewSizeAction {
  return {
    type: ActionType.SET_botRightPanelViewSize,
    botRightPanelViewSize
  }
}

export function setBotLeftPanelViewSize(botLeftPanelViewSize: PanelViewSize): SET_botLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_botLeftPanelViewSize,
    botLeftPanelViewSize
  }
}


//--- dialogs
export function setIsCustomTestDialogDisplayed(isCustomTestDialogDisplayed: boolean): SET_isCustomTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isCustomTestDialogDisplayed,
    isCustomTestDialogDisplayed
  }
}

export function setIsViewTestDialogDisplayed(isViewTestDialogDisplayed: boolean): SET_isViewTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isViewTestDialogDisplayed,
    isViewTestDialogDisplayed
  }
}

export function setIsDoCodeEditorSettingsDialogDisplayed(isDisplayed: boolean): SET_isCodeEditorSettingsDialogDisplayedAction {
  return {
    type: ActionType.SET_isCodeEditorSettingsDialogDisplayed,
    isDisplayed: isDisplayed
  }
}


//--- custom test asset files

export function setIsAddCustomTestAssetFilesModalDisplayed(isDisplayed: boolean, isBusy: boolean): SET_isAddCustomTestAssetFilesModalDisplayedAction {
  return {
    type: ActionType.SET_isAddCustomTestAssetFilesModalDisplayed,
    isDisplayed,
    isBusy
  }
}

export function setAddCustomTestAssetFilesModalFilterText(addCustomTestAssetFilesModalFilterText: string): SET_addCustomTestAssetFilesModalFilterTextAction {
  return {
    type: ActionType.SET_addCustomTestAssetFilesModalFilterText,
    addCustomTestAssetFilesModalFilterText
  }
}

export function setAddCustomTestAssetFilesModalUploadPercentage(addCustomTestAssetFilesModalUploadPercentage: number | null | 'active'): SET_addCustomTestAssetFilesModalUploadPercentageAction {
  return {
    type: ActionType.SET_addCustomTestAssetFilesModalUploadPercentage,
    addCustomTestAssetFilesModalUploadPercentage
  }
}



export function loadDoExerciseSite(releaseCode: string): AwaitActions {
  return async (dispatch, getState) => {

    //resets displayed dialogs & all old states (console output...)
    dispatch(resetDoExerciseSite()) //this also sets the dummy to true


    //clear the old timer
    dispatch(setExercisePreviewFrontendDoExercise(null))

    await dispatch(getGlobalSystemSettingsAsync())

    await dispatch(getTestServerPublicSettingsAsync())

    await dispatch(getTestTypesAsync())

    await dispatch(getPLangsAsync())

    await dispatch(getDoExerciseAsync(releaseCode))

    //this needs to be called BEFORE getDoExerciseSolutionVersion
    //because in getDoExerciseSolutionVersion we check LastKnownDoEditorReleaseId
    FrontendSettingsManager.setLastKnownDoEditorReleaseId(
      FrontendSettingsManager.getFrontendSettings().doEditorReleaseId)
    FrontendSettingsManager.setDoEditorReleaseId(getState().doExerciseSiteState.exercise.exerciseRelease.id)

    const lastEditedPLangId = getState().doExerciseSiteState.exercise.lastEditedPLangId
    await dispatch(getDoExerciseSolutionAsync(releaseCode, lastEditedPLangId))


    //make sure we have editors up-to-date (create AceEditorsStorageHelper tabs for every user solution file)
    if (!AceEditorsStorageHelper.getEditor(doEditorEditorPanelViewEditorId)) {
      const userSolution = getState().doExerciseSiteState.solution
      const pLangs = getState().pLangsState.pLangs
      const plang = pLangs.find(p => p.id === userSolution.pLangId)

      if (!plang) {
        Logger.error('[doExerciseSiteActions] could not find plant for user solution')
      }

      const tabs = userSolution.solutionFiles.map<AceStoredTabLike>(p => {
        return {
          id: p.id,
          content: p.content,
          language: plang.internalName
        }
      })

      AceEditorsStorageHelper.addEditorWithTabs(doEditorEditorPanelViewEditorId, tabs)
    }

    await dispatch(
      getDoExerciseAllTestResults(releaseCode, lastEditedPLangId))

    const solution = getState().doExerciseSiteState.solution
    //try to select the first displayed tab (if any)
    if (solution.solutionFiles.length > 0
      && solution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
      dispatch(setSelectedTabIdDoEditor(
        solution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
    }

    //if the release has ended AND the results are there... get the submit test results...

    if (!getState().doExerciseSiteState.exercise.exerciseRelease.isReleased
      && getState().doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished) {
      await dispatch(getDoExerciseSubmitTestResults(releaseCode, getState().doExerciseSiteState.solution.pLangId))
    }

    //after editor

    if (solution.shouldLoadAfterSolution) {

      if (solution.hasLoadAfterSolution) {
        await dispatch(getDoExerciseAfterSolutionAsync(releaseCode, lastEditedPLangId))


        //make sure we have editors up-to-date (create AceEditorsStorageHelper tabs for every user solution file)
        if (!AceEditorsStorageHelper.getEditor(doEditorAfterEditorPanelViewEditorId)) {
          const userAfterSolution = getState().doExerciseSiteState.afterSolution
          const pLangs = getState().pLangsState.pLangs
          const plang = pLangs.find(p => p.id === userAfterSolution.pLangId)

          if (!plang) {
            Logger.error('[doExerciseSiteActions] could not find plant for user solution')
          }

          const tabs = userAfterSolution.solutionFiles.map<AceStoredTabLike>(p => {
            return {
              id: p.id,
              content: p.content,
              language: plang.internalName
            }
          })

          AceEditorsStorageHelper.addEditorWithTabs(doEditorAfterEditorPanelViewEditorId, tabs)
        }

        if (getState().doExerciseSiteState.afterSolution.solutionFiles.length > 0
          && getState().doExerciseSiteState.afterSolution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
          dispatch(setAfterEditor_selectedTabIdDoEditor(
            getState().doExerciseSiteState.afterSolution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
        }

        await dispatch(getAllDoExerciseAfterTestResults(releaseCode, lastEditedPLangId))

        if (!getState().doExerciseSiteState.exercise.exerciseRelease.isReleased
          && getState().doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished) {
          await dispatch(getDoExerciseAfterSubmitTestResults(releaseCode, lastEditedPLangId))
        }

      }
      else {
        //create the initial dummy after solution
        //this is done via the check if time is up
      }
    }

    //create the assets
    createAndRefreshAssetUrlsAndObjects(getState().doExerciseSiteState.exercise.exerciseDescription.assets)

    //somehow force the preview panel to re render
    //this is also called if the user must re log in
    dispatch(setDoExerciseDummy(false))

  }
}

export function createInitialAfterSolution(): MultiActions {
  return (dispatch, getState) => {

    const releaseCode = getState().doExerciseSiteState.exercise.exerciseRelease.releaseCode
    const lastEditedPLangId = getState().doExerciseSiteState.exercise.lastEditedPLangId

    dispatch(_createFirstDummyAfterSolution(releaseCode, lastEditedPLangId))

    if (getState().doExerciseSiteState.afterSolution.solutionFiles.length > 0
      && getState().doExerciseSiteState.afterSolution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
      dispatch(setAfterEditor_selectedTabIdDoEditor(
        getState().doExerciseSiteState.afterSolution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
    }

    //also focus this editor
    dispatch(setTopLeftSelectedTabIndex(1))

  }
}


/**
 * this is not recommended bcause the solution might not match the required file structure (names, ...)
 * so only call this if you are sure it will work (or on your own risk)
 *
 * CURRENTLY ONLY USED FOR RECOVERY CODE FROM FRONTEND SETTINGS
 * but this is needed for crash recovery for the solution files...
 * BUT it's bad...
 * @param {SolutionDoExerciseFullBase} solution
 * @returns {SET_solutionAction}
 */
export function setSolutionVersion(solution: SolutionDoExerciseFullBase): SET_solutionAction {
  return {
    type: ActionType.SET_solution,
    solution
  }
}

export function setDoExerciseDummy(dummy: boolean): SET_dummyAction {
  return {
    type: ActionType.SET_dummy,
    dummy
  }
}

export function setLastKnownEditorContent_DoExerciseSite(lastKnownEditorContent: string): SET_lastKnownEditorContentAction {
  return {
    type: ActionType.SET_lastKnownEditorContent,
    lastKnownEditorContent
  }
}

export function setJustRunProgramArgsDoEditor(justRunProgramArgs: string): SET_justRunProgramArgsAction {
  return {
    type: ActionType.SET_justRunProgramArgs,
    justRunProgramArgs
  }
}

export function setIsJustRunProgramArgsInputDisplayedDoEditor(isJustRunProgramArgsInputDisplayed: boolean): SET_isJustRunProgramArgsInputDisplayedAction {
  return {
    type: ActionType.SET_isJustRunProgramArgsInputDisplayed,
    isJustRunProgramArgsInputDisplayed
  }
}

export function resetDoExerciseSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}

export function setIsLoadingDoExerciseSite(isLoading: boolean): SET_isLoadingAction {
  return {
    type: ActionType.SET_isLoading,
    isLoading
  }
}

export function setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(isOnGotFocusInterceptionOverlayDisplayed: boolean): SET_isOnGotFocusInterceptionOverlayDisplayedAction {
  return {
    type: ActionType.SET_isOnGotFocusInterceptionOverlayDisplayed,
    isOnGotFocusInterceptionOverlayDisplayed
  }
}


