/**
 * Created by janis dähne
 */

const path = require('path')
const webpack = require('webpack')
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const poststylus = require('poststylus')
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
// const CompressionPlugin = require('compression-webpack-plugin');
// const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;


module.exports = {

  mode: "production", //production not working?? //this already does UglifyJsPlugin...
  entry: {
    app: './src/index.tsx',
  },

  output: {
    publicPath: "/",
    path: path.join(__dirname, 'dist'),
    filename: 'static/[name]_bundle.js', //cache busting hashing is added via HtmlWebpackPlugin
    chunkFilename: 'static/chunks/[name].chunk.js'
  },
  resolve: {
    // Add '.ts' and '.tsx' as resolvable extensions.
    extensions: [".ts", ".tsx", ".js", ".json", ".css"]
  },
  module: {

    rules:  [
      {test: /\.(jpe?g|gif|png|svg|woff|ttf|wav|mp3)$/, loader: "file-loader"}, //|css
      {test: /\.tsx?$/, loader: "awesome-typescript-loader"},

      {enforce: "pre", test: /\.js$/, loader: "source-map-loader"},

      {
        test: /\.(woff|woff2|eot|ttf)$/,
        loader: 'url-loader?limit=100000',
      },

      { test: /\.css$/, use: [MiniCssExtractPlugin.loader, 'css-loader'] }, //import '*.css will be taken into one chunk
      {
        test: /\.styl$/,
        use: ['style-loader', MiniCssExtractPlugin.loader, 'css-loader', 'stylus-loader']
      },
      {
        test: /\.md$/,
        loader: 'raw-loader'
      }
    ]
  },
  optimization: {
    splitChunks: {
      chunks: 'all', //see https://medium.com/dailyjs/webpack-4-splitchunks-plugin-d9fbbe091fd0
      minSize: 30000,
      maxSize: 0,
      minChunks: 1,
      maxAsyncRequests: 5,
      maxInitialRequests: 3,
      automaticNameDelimiter: '~', //vendors~app.chunk.js means that app.chunk.js depends/uses vendor.js
      name: true,
      cacheGroups: {
        vendors: {
          test: /[\\/]node_modules[\\/]/,
          priority: -10
        },
        default: {
          minChunks: 2,
          priority: -20,
          reuseExistingChunk: true
        }
      }
    }
  },
  plugins: [
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': '"production"'
    }),
    new webpack.ContextReplacementPlugin(
      /moment[\/\\]locale$/,
      /de|en/
    ),
    new webpack.LoaderOptionsPlugin({
      options: {
        stylus: {
          use: [poststylus([ 'autoprefixer'])]
        }
      }
    }),
    new webpack.optimize.ModuleConcatenationPlugin(),
    new CopyWebpackPlugin([
      {
        from: 'node_modules/react/dist/react.min.js',
        to: 'static/react.js',
      },
      {
        from: 'node_modules/react-dom/dist/react-dom.min.js',
        to: 'static/react-dom.js',
      },
      {
        from: 'node_modules/semantic-ui-offline/semantic.min.css',
        to: 'static/semantic.min.css',
      },
      {
        from: 'node_modules/semantic-ui-offline/font',
        to: 'static/font',
      },
      {
        from: 'node_modules/semantic-ui-offline/themes',
        to: 'static/themes',
      },
      {
        from: 'node_modules/brace/mode',
        to: 'static/ace-modes',
        transformPath(targetPath, absolutePathFrom) {
          return `static/ace-modes/mode-${path.basename(targetPath)}`
        }
      },
      //copy math jax stuff (only the minimal needed to render stuff as we no longer use cdn...)
      {
        from: 'node_modules/mathjax/MathJax.js',
        to: 'static/mathjax/MathJax.js',
      },
      {
        from: 'node_modules/mathjax/config/Safe.js',
        to: 'static/mathjax/config/Safe.js',
      },
      {
        from: 'node_modules/mathjax/config/TeX-MML-AM_CHTML.js',
        to: 'static/mathjax/config/TeX-MML-AM_CHTML.js',
      },
      {
        from: 'node_modules/mathjax/extensions/MathMenu.js',
        to: 'static/mathjax/extensions/MathMenu.js',
      },
      {
        from: 'node_modules/mathjax/extensions/MathZoom.js',
        to: 'static/mathjax/extensions/MathZoom.js',
      },
      {
        from: 'node_modules/mathjax/extensions/Safe.js',
        to: 'static/mathjax/extensions/Safe.js',
      },
      {
        from: 'node_modules/mathjax/extensions/tex2jax.js',
        to: 'static/mathjax/extensions/tex2jax.js',
      },
      {
        from: 'node_modules/mathjax/fonts/HTML-CSS/TeX',
        to: 'static/mathjax/fonts/HTML-CSS/TeX',
      },
      {
        from: 'node_modules/mathjax/jax/output/CommonHTML',
        to: 'static/mathjax/jax/output/CommonHTML',
      },
      //--- copy also static pages & assets
      {
        from: 'static_pages/static/sha512.js', //needed for firstLoginExternalLogin
        to: 'static/sha512.js',
      },
      {
        from: 'static_pages/static/favicon.png',
        to: 'favicon.png',
      },
      {
        from: 'static_pages/static/about_icon.png',
        to: 'about_icon.png',
      },
      //hack because react-notifications cannot properly bundled ... the icons are not displayed
      //TODO not working because these files are overwritten by the loader!!
      {
        from: 'node_modules/react-notifications/dist/fonts/notification.ttf',
        to: 'a8c8792074cb2979091029707dbf0c46.ttf',
      },
      {
        from: 'node_modules/react-notifications/dist/fonts/notification.woff',
        to: '80abd98351a79a932614527b220f71e8.woff',
      },
      //--- static pages
      {
        from: 'static_pages/about/about.html',
        to: 'about/about.html',
      },
      {
        from: 'static_pages/about/about.js',
        to: 'about/about.js',
      },
      {
        from: 'static_pages/about/about.css',
        to: 'about/about.css',
      },
      {
        from: 'static_pages/about/dependencies/',
        to: 'about/dependencies',
        toType: 'dir'
      },
      {
        from: 'static_pages/about/imgs/',
        to: 'about/imgs',
        toType: 'dir'
      },
      {
        from: 'static_pages/about/screenshots/',
        to: 'about/screenshots',
        toType: 'dir'
      },
      {
        from: 'static_pages/copyright/copyright.html',
        to: 'copyright/copyright.html',
      },
      {
        from: 'static_pages/privacyPolicy/privacy.html',
        to: 'privacypolicy/privacy.html',
      },
      {
        from: 'static_pages/firstExternalLoginPage/firstLoginExternalLogin.html',
        to: 'firstLoginExternalLogin.html',
      },
      {
        from: 'static_pages/firstExternalLoginPage/firstLoginExternalLogin.js',
        to: 'firstLoginExternalLogin.js',
      },
    ]),
    new MiniCssExtractPlugin({
      filename: 'static/styles.css'
    }),
    new HtmlWebpackPlugin({
      hash: true,
      filename: 'index_deploy.html',
      template: 'htmlTemplates/index_deploy.ejs',
      inject: false,
      cache: false,
      //this is only for the template file
      minify: {
        caseSensitive: true,
        minifyCSS: true,
        collapseWhitespace: false,
        removeAttributeQuotes: true,
        removeComments: true
      }
    }),
    // new CompressionPlugin({ //already donce on server via nginx?
    //   cache: true
    // })
    //enable to get a nice treemap showing all sizes
    // new BundleAnalyzerPlugin()
  ],
  externals: {
    //we include these via script tag... more parallel loads?
    "react": "React",
    "react-dom": "ReactDOM"
  }

}


